#version 330 core
#extension GL_ARB_shading_language_include : require

// based on http://www.opengl-tutorial.org, WTFPL
// features lighting additions from https://learnopengl.com/, CC BY-NC 4.0

struct Material {
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    float shininess;
};
uniform Material material;

uniform vec3 lightPos;
uniform vec3 objColour;

in vec3 normal;

in vec3 pos_ws;
in vec3 normal_cs;
in vec3 lightdir_cs;
in vec3 eyedir_cs;

out vec4 FragColor;

void main()
{
    vec3 lightColour = vec3(1, 1, 1);
    float lightPower = 200.0;

    vec3 n = normalize( normal_cs );
    vec3 l = normalize( lightdir_cs );
    float distance = length( lightPos - pos_ws );

    vec3 e = normalize( eyedir_cs );
    vec3 r = reflect( -l, n );
    float cosAlpha = clamp( dot(e, r), 0, 1);
    float cosTheta = clamp( dot( n, l ), 0, 1 );

    vec3 colour =
       ( material.ambient * vec3(0.1) ) +
       ( material.diffuse * vec3(0.5) * lightColour * lightPower * cosTheta / ( distance*distance ) ) +
       ( material.specular * lightColour * lightPower * pow( cosAlpha, material.shininess * 128 ) / (distance*distance) );

    FragColor = vec4(colour, 1);
} 
