/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#include <cstdlib>
#include <memory>

#include <spdlog/spdlog.h>
#include <fggl/app.hpp>

namespace fggl {

	App::App(modules::Manager *services, const Identifer &name) : App::App(services, name, name) {
	}

	App::App(modules::Manager *services, const Identifer & /*name*/, const Identifer & /*folder*/ ) :
		m_running(true),
		m_window(nullptr),
		m_states(),
		m_subsystems(services) {}

	int App::run(int argc, const char **argv) {
		auto *windowing = m_subsystems->get<display::WindowService>();

		{
			// activate the first state
			auto &state = m_states.active();
			m_expectedScene = m_states.activeID();
			state.activate();
		}

		while (m_running) {
			// trigger a state change if expected
			if (m_expectedScene != m_states.activeID()) {
				auto result = m_states.change(m_expectedScene);
				if (!result) {
					m_expectedScene = m_states.activeID();
				}
			}

			// process window events
			if (windowing != nullptr) {
				windowing->pollEvents();
			}
			//m_modules->onUpdate();

			auto &state = m_states.active();
			state.update();

			// window rendering to frame buffer
			if (m_window != nullptr) {
				//	m_modules->onFrameStart();
				m_window->frameStart();

				// get draw instructions
				auto &graphics = m_window->graphics();
				state.render(graphics);

				m_window->frameEnd();
				//  m_modules->onFrameEnd();

				m_running = m_running && !m_window->wantClose();
			}
		}

		{
			// shutdown last state
			auto &state = m_states.active();
			state.deactivate();
		}

		return EXIT_SUCCESS;
	}

} //namespace fggl

