/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#include <fggl/data/model.hpp>

#include <map>
#include <stdexcept>
#include <algorithm>

using namespace fggl::data;

Mesh::Mesh() : restartVertex(Mesh::INVALID_IDX), m_verts(0), m_index(0) {
}

void Mesh::pushIndex(unsigned int idx) {
	assert(idx < m_verts.size() || idx == this->restartVertex);
	m_index.push_back(idx);
}

Mesh::IndexType Mesh::pushVertex(Vertex vert) {
	auto idx = m_verts.size();
	m_verts.push_back(vert);
	return idx;
}

Mesh::IndexType Mesh::indexOf(Vertex term) {
	auto itr = std::find(m_verts.begin(), m_verts.end(), term);
	if (itr == m_verts.end()) {
		return INVALID_IDX;
	}
	return itr - m_verts.begin();
}

void Mesh::removeDups() {
	// lookups to make detecting duplicates easier
	std::map<Vertex, unsigned int> mapping;

	// new data
	std::vector<Vertex> newVerts;
	std::vector<unsigned int> newIndexes;
	newIndexes.reserve(m_index.size()); // newIndex will be same size as oldIndex

	unsigned int nextID = 0;
	for (auto &idx : m_index) {
		auto &vertex = m_verts[idx];
		auto newID = nextID;

		// if the vertex is known, use the existing ID, else allocate a new ID
		try {
			newID = mapping.at(vertex);
		} catch (std::out_of_range &e) {
			mapping[vertex] = newID;
			newVerts.push_back(vertex);
			++nextID;
		}
		newIndexes.push_back(newID);
	}

	// assign the replacement lists
	m_verts = newVerts;
	m_index = newIndexes;
}

