/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#include <fggl/gui/containers.hpp>
#include <spdlog/spdlog.h>

namespace fggl::gui {

	Widget *Container::getChildAt(const math::vec2 &point) {
		for (auto &child : m_children) {
			if (child->contains(point)) {
				return child->getChildAt(point);
			}
		}

		return nullptr;
	}

	bool Container::contains(const math::vec2 &point) {
		return true;
	}

	void Container::render(gfx::Paint &paint) {
		for (auto &child : m_children) {
			child->render(paint);
		}
	}

	void Container::add(std::unique_ptr<Widget> widget) {
		m_children.push_back(std::move(widget));
		m_dirty = true;
	}

	/*
	Box::Box( LayoutAxis axis ) : m_axis( axis ) {}
	
	void Box::layout() {
		math::vec2 baseSize{};
		
		const int mainAxis = 0;
		const int crossAxis = 1;
		
		// ask for preferred sizes
		for ( auto& child : m_children ) {
			auto childBaseSize = child.baseSize();
			
			baseSize[mainAxis] += childBaseSize[mainAxis];
			if ( childBaseSize[crossAxis] > baseSize[crossAxis] ) {
				baseSize[crossAxis] = childBaseSize[crossAxis];			
			}
		}
		
		if ( lineAxis == 0 )
			setSize(lineSum, biggestCross);
		else
			setSize(biggestCross, lineSum);
	}*/

	void Panel::render(gfx::Paint &paint) {
		// background painting time
		gfx::Path2D background(topLeft());
		background.colour(math::vec3(32.0f / 255.0f, 74.0F / 255.0F, 135.0F / 255.0F));
		draw_box(background, topLeft(), bottomRight());
		paint.fill(background);

		// call parent's render method for child objects
		Container::render(paint);
	}
};
