/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_AUDIO_AUDIO_HPP
#define FGGL_AUDIO_AUDIO_HPP

#include <string>
#include "fggl/data/storage.hpp"
#include "fggl/modules/module.hpp"

//! backend independent audio interface
namespace fggl::audio {

	/**
	 * AudioClip is bit of audio loaded into memory.
	 *
	 * If the sampleCount is -1, the clip is invalid.
	 */
	struct AudioClip {
		int channels = 0;
		int sampleRate = 0;
		int sampleCount = -1;
		short *data = nullptr;
	};

	constexpr modules::ModuleService SERVICE_AUDIO_PLAYBACK = modules::make_service("fggl::audio::AudioService");

	/**
	 *
	 * \ingroup services
	 */
	class AudioService {
		public:
			constexpr static const modules::ModuleService service = SERVICE_AUDIO_PLAYBACK;
			virtual void play(const std::string &filename, bool looping = false) = 0;
			virtual void play(AudioClip &clip, bool looping = false) = 0;

			virtual ~AudioService() = default;
	};

} // namespace fggl::audio

#endif //FGGL_AUDIO_AUDIO_HPP
