/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 20/08/22.
//

#ifndef FGGL_PHYS_NULL_HPP
#define FGGL_PHYS_NULL_HPP

#include "fggl/phys/types.hpp"
#include "fggl/phys/service.hpp"

namespace fggl::phys {

	inline void build_noop(const entity::ComponentSpec & /*config*/,
						   entity::EntityManager & /*manager*/,
						   const entity::EntityID & /*entity*/) {}

	class NullPhysicsEngine : public PhysicsEngine {
		public:
			inline void step() override {}

			void setDebugDraw(bool /* enable */) override {}

			inline std::vector<ContactPoint> scanCollisions(entity::EntityID /*entity*/) override {
				return {};
			}

			inline entity::EntityID raycast(math::vec3 /*from*/, math::vec3 /*to*/) override {
				return entity::INVALID;
			}

			inline std::vector<entity::EntityID> raycastAll(math::vec3 /*from*/, math::vec3 /*to*/) override {
				return {};
			}

			inline std::vector<entity::EntityID> sweep(PhyShape & /*shape*/,
													   math::Transform & /*from*/,
													   math::Transform & /*to*/) override {
				return {};
			}
	};

	class NullPhysicsProvider : public PhysicsProvider {
		public:
			virtual ~NullPhysicsProvider() = default;

			inline PhysicsEngine *create(entity::EntityManager * /*entityManager*/,
										 entity::EntityFactory *factory) override {
				factory->bind(util::make_guid("phys::Body"), build_noop);
				return new NullPhysicsEngine();
			}
	};

	struct NullPhysics {
		constexpr static const char *name = "fggl::phys::null";
		constexpr static const std::array<modules::ModuleService, 1> provides = {
			phys::PhysicsProvider::service
		};
		constexpr static const std::array<modules::ModuleService, 1> depends = {
			entity::EntityFactory::service
		};
		static bool factory(modules::ModuleService serviceName, modules::Services &serviceManager);
	};

} // namespace fggl::phys

#endif //FGGL_PHYS_NULL_HPP
